#!/bin/ksh
# ******************************************************************************************************************************
# * Licensed Materials - Property of IBM                                                                                       *
# * (c) Copyright IBM Corp. 2008, 2009 All Rights Reserved.                                                                    *
# *                                                                                                                            *
# *US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp*
# ******************************************************************************************************************************

rcrdone(){
db2 connect reset
db2 terminate
echo $MESSAGE:$EXITCODE
echo $CRREWECH002$
exit $EXITCODE
}

chmod -R +x lang

if [ `echo $LANG | sed 's/\([a-z]*\).*/\1/'` == "en" ]
then 
     LOCALE="en"
     SQLVARF="utf8/vars_en.sql"
	. lang/en/messages.sh
fi	

if [ `echo $LANG | sed 's/\([a-z]*\).*/\1/'` == "it" ]
then 
	LOCALE="it"
	SQLVARF="utf8/vars_it.sql"
	. lang/it/messages.sh
fi	

if [ `echo $LANG | sed 's/\([a-z]*\).*/\1/'` == "pl" ]
then 
	LOCALE="pl"
	SQLVARF="utf8/vars_pl.sql"
	. lang/pl/messages.sh
fi	

if [ `echo $LANG | sed 's/\([a-z]*\).*/\1/'` == "ko" ]
then 
	LOCALE="kr"
	SQLVARF="utf8/vars_kr.sql"
	. lang/kr/messages.sh
fi

if [ `echo $LANG | sed 's/\([a-z]*\).*/\1/'` == "ja" ]
then 
	LOCALE="ja"
	SQLVARF="utf8/vars_ja.sql"
	. lang/ja/messages.sh
fi

if [ `echo $LANG | sed 's/\([a-z]*_[A-Z][A-Z]\).*/\1/'` == "zh_CN" ]
then 
	LOCALE="zh_CN"
	SQLVARF="utf8/vars_zh_CN.sql"
	. lang/zh_CN/messages.sh
fi

if [ `echo $LANG | sed 's/\([a-z]*_[A-Z][A-Z]\).*/\1/'` == "zh_TW" ]
then 
	LOCALE="zh_TW"
	SQLVARF="utf8/vars_zh_TW.sql"
	. lang/zh_TW/messages.sh
fi

if [ -z $LOCALE ]
then
	 LOCALE="en"
	 SQLVARF="utf8/vars_en.sql"
	. lang/en/messages.sh
fi	


SILENT=0
MESSAGE="No errors"
EXITCODE=0

DB2_CMD="db2 -tsf"

if [ $# -eq 1 ]
	then 
	if [ $1 == "-silent" ]
	then
			SILENT=1
	else 
			SILENT=0
	fi
	if [ $1 == "-debug" ]
	then
		DB2_CMD="db2 -tvsf"
		else 
			DB2_CMD="db2 -tsf"
	fi
fi

if [ $SILENT -eq 0 ]
then 
	echo $CRREWECH033
fi	

if [ $SILENT -eq 0 ]
then 
	echo $CRREWPRMPT028
	read DBNAME
fi

if [ -z $DBNAME ]
then
	MESSAGE=$CRREWMSGE002
	EXITCODE=$CRREW4502E
	rcrdone
fi

if [ $SILENT -eq 0 ]
then 
	echo $CRREWPRMPT003
	read DBUSER
fi

if [ -z $DBUSER ]
then
	MESSAGE=$CRREWMSGE003
	EXITCODE=$CRREW4503E
	rcrdone
fi

if [ $SILENT -eq 0 ]
then 
	echo $CRREWPRMPT004
	read DBPASSWD
fi

if [ -z $DBPASSWD ]
then
	MESSAGE=$CRREWMSGE004
	EXITCODE=$CRREW4504E
	rcrdone
fi

if [ $SILENT -eq 0 ]
then 
	echo $CRREWPRMPT029
	read NF_SCHEMA
fi

if [ -z $NF_SCHEMA ]
then
	MESSAGE=$CRREWMSGE005
	EXITCODE=$CRREW4505E
	rcrdone
fi

if [ $SILENT -eq 0 ]
then 
	echo $CRREWPRMPT030
	read STAR_SCHEMA
fi

if [ -z $STAR_SCHEMA ]
then
	MESSAGE=$CRREWMSGE006
	EXITCODE=$CRREW4506E
	rcrdone
fi

if [ $SILENT -eq 0 ]
then 
	echo $CRREWPRMPT031
	read REP_USER
fi

if [ -z $REP_USER ]
then
	MESSAGE=$CRREWMSGE007
	EXITCODE=$CRREW4507E
	rcrdone
fi

if [ $DBUSER == $REP_USER ]
then
	EXITCODE=$CRREW4595E
	MESSAGE=$CRREWMSGE095
	rcrdone
fi

if [ $SILENT -eq 0 ]
then 
	echo $CRREWPRMPT032
	read VIEW_REDEF
fi

if [ -z $VIEW_REDEF ]
then
    VIEW_REDEF=0
fi

if [ $VIEW_REDEF -ne 1 ]
then
    VIEW_REDEF=0
fi    

CONNSTR="db2 CONNECT TO $DBNAME USER $DBUSER USING $DBPASSWD"

CD=`pwd`

if [ ! -d $CD/work_spc ]
then
	mkdir $CD/work_spc
else
	rm -fr $CD/work_spc
	mkdir $CD/work_spc
fi	
if [ $? -ne 0 ]
then
	MESSAGE=$CRREWMSGE076
	EXITCODE=$CRREW4576E
	rcrdone
fi
    

if [ ! -d $CD/logs ]
then
	mkdir $CD/logs
else
	rm -fr $CD/logs
	mkdir $CD/logs	
fi
if [ $? -ne 0 ]
then
	MESSAGE=$CRREWMSGE076
	EXITCODE=$CRREW4576E
	rcrdone
fi

cp -r ./ddl $CD/work_spc
if [ $? -ne 0 ]
then
	MESSAGE=$CRREWMSGE024
	EXITCODE=$CRREW4524E
	rcrdone
fi

cp -r ./scripts $CD/work_spc
if [ $? -ne 0 ]
then
	MESSAGE=$CRREWMSGE024
	EXITCODE=$CRREW4524E
	rcrdone
fi

$CONNSTR
db2 "INSERT INTO CONFIG.SYSCONFIG_INFO(INFO_ID,NAME,DISPLAY_NAME,INT_VALUE,CHAR_VALUE)SELECT 6 AS INFO_ID,'VERSION' AS NAME,'Insight Warehouse Version' AS DISPLAY_NAME,1 AS INT_VALUE,'1.0' AS CHAR_VALUE FROM SYSIBM.SYSDUMMY1 WHERE NOT EXISTS(SELECT INFO_ID FROM CONFIG.SYSCONFIG_INFO WHERE INFO_ID=6)"

db2 "INSERT INTO CONFIG.SYSCONFIG_INFO(INFO_ID,NAME,DISPLAY_NAME,INT_VALUE,CHAR_VALUE) SELECT 7 AS INFO_ID,'UPDATE_TARGET' AS NAME,'Insight Warehouse Update' AS DISPLAY_NAME,0 AS INT_VALUE,'1.0.0.2' AS CHAR_VALUE FROM SYSIBM.SYSDUMMY1 WHERE NOT EXISTS(SELECT INFO_ID FROM CONFIG.SYSCONFIG_INFO WHERE INFO_ID=7)"

db2 "UPDATE CONFIG.SYSCONFIG_INFO SET CHAR_VALUE='1.0.0.2' WHERE INFO_ID=7"

EXISTS=`$CONNSTR; db2 -x + -o - "SELECT COUNT(*) FROM $STAR_SCHEMA.CAL_MESSAGES WHERE CAL_MESSAGE_CODE IN ('CRREW4598E','CRREW4599E','CRREW4600E','CRREW4601E','CRREW4602E')"`
EXISTS=`echo $EXISTS|rev|cut -c 1`

if [ $EXISTS -eq 0 ]
then
grep -e "CRREW4598E" ./work_spc/scripts/$SQLVARF>>work_spc/scripts/messages.sql
grep -e "CRREW4599E" ./work_spc/scripts/$SQLVARF>>work_spc/scripts/messages.sql
grep -e "CRREW4600E" ./work_spc/scripts/$SQLVARF>>work_spc/scripts/messages.sql
grep -e "CRREW4601E" ./work_spc/scripts/$SQLVARF>>work_spc/scripts/messages.sql
grep -e "CRREW4602E" ./work_spc/scripts/$SQLVARF>>work_spc/scripts/messages.sql

$CONNSTR
db2 "SET CURRENT SCHEMA $STAR_SCHEMA"
$DB2_CMD ./work_spc/scripts/messages.sql>./logs/messages.log
db2 connect reset
grep -e " SQLSTATE" $CD/logs/messages.log>/dev/null
if [ $? -eq 0 ]
then
	MESSAGE=$CRREWMSGE025 
	EXITCODE=$CRREW4525E 
	rcrdone
fi
fi

EXISTS=`$CONNSTR; db2 -x + -o - "SELECT COUNT(*) FROM SYSCAT.PROCEDURES WHERE PROCNAME IN ('DW_MIGRATE','MIGRATE_MODULE','PROCESS_UOW','VALIDATE_EXISTENCE')"`
EXISTS=`echo $EXISTS|rev|cut -c 1`

if [ $EXISTS -ne 0 ]
then
$CONNSTR
db2 "drop procedure DW_MIGRATE"
db2 "drop procedure MIGRATE_MODULE"
db2 "drop procedure PROCESS_UOW"
db2 "drop procedure VALIDATE_EXISTENCE"
db2 "drop function REPLACE_SCHEMA"
fi

EXISTS=`$CONNSTR; db2 -x + -o - "SELECT COUNT(*) FROM SYSCAT.TABLES WHERE TABNAME IN ('MODULES','STATEMENT_LIST','STATEMENT_DEPENDENCIES')"`
EXISTS=`echo $EXISTS|rev|cut -c 1`

if [ $EXISTS -eq 0 ]
then
$DB2_CMD ./work_spc/ddl/createMIG.ddl> ./logs/createMIG.log
grep -e " SQLSTATE" $CD/logs/createMIG.log>/dev/null
if [ $? -eq 0 ]
then
	MESSAGE=$CRREWMSGE016
	EXITCODE=$CRREW4516E
	echo $CRREWECH006
	rcrdone
fi
fi

$CONNSTR
$DB2_CMD ./work_spc/scripts/statements.sql> ./logs/statements.log
grep -e " SQLSTATE" $CD/logs/statements.log>/dev/null
if [ $? -eq 0 ]
then
	MESSAGE=$CRREWMSGE020
	EXITCODE=$CRREW4520E
	echo $CRREWECH010
	rcrdone
fi

if [ $VIEW_REDEF -eq 1 ]
then
   $CONNSTR
   db2 "UPDATE CONFIG.STATEMENT_LIST SET EXECUTE=1 WHERE RTRIM(ACTION)='ALTER VIEW'">/dev/null
fi



$CONNSTR

db2 -td@ -vf ./work_spc/scripts/migration_sp.sql> ./logs/migration_sp.log

$CONNSTR

db2 "ALTER TABLE $NF_SCHEMA.BUILD ALTER COLUMN BUILDING_TIME SET DATA TYPE  BIGINT"              
db2 "ALTER TABLE $NF_SCHEMA.BUILD ALTER COLUMN WAITING_TIME SET DATA TYPE  BIGINT"               
db2 "REORG TABLE $NF_SCHEMA.BUILD"                                  
db2 "ALTER TABLE $NF_SCHEMA.PROJECT_BASELINE ALTER COLUMN EXTERNAL_KEY2 SET DATA TYPE  CHAR(80)" 
db2 "REORG TABLE $NF_SCHEMA.PROJECT_BASELINE" 

$CONNSTR
db2 SET CURRENT SCHEMA $DBUSER
db2 "CALL $DBUSER.DW_MIGRATE('$NF_SCHEMA','$STAR_SCHEMA','$REP_USER')" >./logs/migrateDB.log
if [ $? -ne 0 ]
then
	MESSAGE=$CRREWMSGE103 
	EXITCODE=$CRREW4525E 
	echo $CRREW4603E
	rcrdone
fi


rcrdone

