--******************************************************************************************************************************
--* Licensed Materials - Property of IBM                                                                                       *
--* (c) Copyright IBM Corp. 2008, 2009 All Rights Reserved.                                                                    *
--*                                                                                                                            *
--*US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp*
--******************************************************************************************************************************


CREATE FUNCTION REPLACE_SCHEMA (STATEMENT VARCHAR(8192), ODS_SCHEMA VARCHAR(20),DM_SCHEMA VARCHAR(20),REP_USER VARCHAR(20))
RETURNS VARCHAR(8192)
NO EXTERNAL ACTION 
READS SQL DATA
BEGIN ATOMIC
		DECLARE	v_text 										VARCHAR(8192);
		DECLARE	v_ods_schema 							VARCHAR(20);
		DECLARE	v_dm_schema 							VARCHAR(20);
		DECLARE v_rep_user								VARCHAR(20);
		
		SET v_text=STATEMENT;
		SET v_ods_schema=ODS_SCHEMA|| '.';
		SET v_dm_schema=DM_SCHEMA || '.';
		SET v_rep_user=REP_USER;
		
		SET v_text=COALESCE(REPLACE(v_text,'RIDW.',v_dm_schema),v_text);
		SET v_text=COALESCE(REPLACE(v_text,'RIODS.',v_ods_schema),v_text);
		SET v_text=COALESCE(REPLACE(v_text,'REPUSER',v_rep_user),v_text);
		RETURN v_text;
END@

CREATE PROCEDURE PROCESS_UOW(MODULE_ID INTEGER,SUB_MODULE_ID INTEGER,UOW_ID INTEGER,ODS_SCHEMA VARCHAR(20),DM_SCHEMA VARCHAR(20), TARGET_VERSION VARCHAR(10),REP_USER VARCHAR(20))
RESULT SETS 1
LANGUAGE SQL     
BEGIN
			DECLARE SQLSTATE CHAR(5) DEFAULT '00000';  
			DECLARE SQLCODE INTEGER DEFAULT 0;
			DECLARE MESSAGE_TEXT VARCHAR(100);
			DECLARE RETVAL INTEGER DEFAULT 0; 
			DECLARE v_sqlstate CHAR(5) DEFAULT '00000';
    		DECLARE v_sqlcode INTEGER DEFAULT 0;
    		DECLARE v_errmsg VARCHAR(100) DEFAULT '';
    		DECLARE err_stmt VARCHAR(1000) DEFAULT '';
    	
    		DECLARE	v_module_id 							INTEGER;
			DECLARE	v_sub_module_id 					INTEGER;
      		DECLARE	v_uow_id 									INTEGER;
      		DECLARE v_ods_schema VARCHAR(20); 
			DECLARE v_dm_schema VARCHAR(20);
			
			DECLARE stmt VARCHAR(1000) DEFAULT ''; 
			DECLARE	v_statement_number 				INTEGER;
			DECLARE	v_area 										VARCHAR(255);
			DECLARE	v_type 										VARCHAR(255);
			DECLARE	v_action 									VARCHAR(255);
			DECLARE	v_data_movement 					INTEGER;
			DECLARE	v_table_refactoring 			INTEGER;
			DECLARE	v_description 						VARCHAR(1000);
			DECLARE	v_affected_table 					VARCHAR(255);
			DECLARE v_affected_schema 				VARCHAR(255);
			DECLARE	v_object_type 						VARCHAR(255);
			DECLARE	v_object_name 						VARCHAR(255);
			DECLARE	v_pre_exists 							INTEGER;
			DECLARE	v_recoverable 						INTEGER;
			DECLARE	v_has_dependencies 				INTEGER;
			DECLARE	v_execution_status 				INTEGER;
			DECLARE	v_execute 								INTEGER;
			DECLARE	v_release 								VARCHAR(255);
			DECLARE	v_special_release 				VARCHAR(255);
			DECLARE	v_text 										VARCHAR(8192);
			DECLARE	v_temp_text 							VARCHAR(8192);
			DECLARE v_failled_pred integer DEFAULT 0;
			DECLARE  v_target_version VARCHAR(255);
			DECLARE v_rep_user VARCHAR(20);
						                                                           
      						
			DECLARE EXIT HANDLER FOR SQLSTATE '70010'                              
			BEGIN 
			  ROLLBACK;  
	      UPDATE CONFIG.STATEMENT_LIST SET EXECUTION_STATUS=0,RET_CODE=v_sqlstate WHERE STATEMENT_NUMBER=v_statement_number;
	      COMMIT; 
				SET err_stmt='VALUES (' || v_dm_schema || '.SELECT_CAL_MESSAGE(''CRREW4601E''))';
				PREPARE s_err7 FROM err_stmt;
				BEGIN
					DECLARE CE6 CURSOR WITH HOLD FOR s_err7;
					OPEN CE6;
					FETCH CE6 INTO v_errmsg;
					CLOSE CE6;
				END;                                                                                                                       
				RESIGNAL SQLSTATE '70010' SET MESSAGE_TEXT = v_errmsg;                                                                 
			END;
			
			
			DECLARE EXIT HANDLER FOR SQLSTATE '70011'                              
			BEGIN 
			  ROLLBACK;  
	      UPDATE CONFIG.STATEMENT_LIST SET EXECUTION_STATUS=0,RET_CODE='70011' WHERE STATEMENT_NUMBER=v_statement_number;
	      COMMIT; 
				SET err_stmt='VALUES (' || v_dm_schema || '.SELECT_CAL_MESSAGE(''CRREW4602E''))';
				PREPARE s_err8 FROM err_stmt;
				BEGIN
					DECLARE CE7 CURSOR WITH HOLD FOR s_err8;
					OPEN CE7;
					FETCH CE7 INTO v_errmsg;
					CLOSE CE7;
				END;                                                                                                                       
				RESIGNAL SQLSTATE '70011' SET MESSAGE_TEXT = v_errmsg;                                                                 
			END;
			
			SET v_module_id=MODULE_ID;
			SET v_sub_module_id=SUB_MODULE_ID;
			SET v_uow_id = UOW_ID;
			SET v_ods_schema= ODS_SCHEMA;
			SET v_dm_schema = DM_SCHEMA;
			SET v_target_version= TARGET_VERSION;
			SET v_rep_user=REP_USER;
			
		
			BEGIN
			DECLARE C4 CURSOR WITH HOLD FOR 
				SELECT STATEMENT_NUMBER,AREA,DATA_MOVEMENT,TABLE_REFACTORING,AFFECTED_TABLE,
				AFFECTED_SCHEMA,OBJECT_TYPE,OBJECT_NAME,RECOVERABLE,HAS_DEPENDENCIES,TEXT
				FROM CONFIG.STATEMENT_LIST 
				WHERE MODULE_ID=v_module_id AND SUB_MODULE_ID=v_sub_module_id AND UOW_ID=v_uow_id
				AND EXECUTE=1 AND (EXECUTION_STATUS IS NULL OR EXECUTION_STATUS<>1) AND RELEASE=v_target_version AND PRE_EXISTS=0
				ORDER BY STATEMENT_NUMBER,UOW_ID;
				DECLARE CONTINUE HANDLER FOR SQLEXCEPTION 
				  BEGIN
					  SET v_sqlstate=SQLSTATE;
					  SET v_sqlcode=SQLCODE;
					  SIGNAL SQLSTATE '70010';
      	  END;

				OPEN C4;

				FETCH C4 INTO v_statement_number,v_area,v_data_movement,v_table_refactoring,v_affected_table,v_affected_schema,v_object_type,v_object_name,v_recoverable,v_has_dependencies,v_text;

				WHILE (SQLSTATE='00000') DO
				  
				  IF (v_has_dependencies=1) THEN
				  SELECT COUNT(*) INTO v_failled_pred FROM CONFIG.STATEMENT_DEPENDENCIES WHERE STATEMENT_NUMBER=v_statement_number AND EXISTS(SELECT STATEMENT_NUMBER FROM CONFIG.STATEMENT_LIST WHERE 
				  STATEMENT_NUMBER=PREDECESSOR_NUMBER AND (EXECUTION_STATUS <>1 OR EXECUTION_STATUS IS NULL)AND PRE_EXISTS<>1);
					  IF(v_failled_pred>0) THEN
						SIGNAL SQLSTATE '70011' ;
					  END IF;
				  END IF;
				  
					SET v_temp_text=REPLACE_SCHEMA(v_text, v_ods_schema,v_dm_schema,v_rep_user);
					PREPARE s4 FROM v_temp_text;
					EXECUTE S4;
					

					IF (SQLSTATE<>'00000') THEN
					BEGIN
					SET v_sqlstate=SQLSTATE;
					SIGNAL SQLSTATE '70010';
					END;
				  ELSE
				  BEGIN
				  	UPDATE CONFIG.STATEMENT_LIST SET EXECUTION_STATUS=1, RET_CODE='00000' WHERE STATEMENT_NUMBER=v_statement_number;
				  	COMMIT;
					END;
				  END IF;
				   FETCH C4 INTO v_statement_number,v_area,v_data_movement,v_table_refactoring,v_affected_table,v_affected_schema,v_object_type,v_object_name,v_recoverable,v_has_dependencies,v_text;  
				 
          END WHILE;
					
				CLOSE C4;
					
				
		 END;
		 RETURN SQLCODE;
      
END@

CREATE PROCEDURE VALIDATE_EXISTENCE(STATEMENT_NUMBER INTEGER, TYPE VARCHAR(255),ACTION VARCHAR(255),AFFECTED_TABLE VARCHAR(255),AFFECTED_SCHEMA VARCHAR(255),OBJECT_TYPE VARCHAR(255),OBJECT_NAME VARCHAR(255),ODS_SCHEMA VARCHAR(20),DM_SCHEMA VARCHAR(20))
RESULT SETS 1
LANGUAGE SQL     
BEGIN
			DECLARE SQLSTATE CHAR(5) DEFAULT '00000';         
			DECLARE SQLCODE INTEGER DEFAULT 0;
			DECLARE MESSAGE_TEXT VARCHAR(100);
			DECLARE RETVAL INTEGER DEFAULT 0; 
			DECLARE v_sqlstate CHAR(5) DEFAULT '00000';
    	DECLARE v_rise INTEGER DEFAULT 0;
    	DECLARE v_errmsg VARCHAR(100) DEFAULT '';
    	DECLARE err_stmt VARCHAR(1000) DEFAULT '';
    	DECLARE v_exists INTEGER DEFAULT 0;
    	DECLARE v_target_version VARCHAR(10);
    	DECLARE v_module INTEGER;
			DECLARE v_sub_module INTEGER;
			DECLARE v_ods_schema VARCHAR(20); 
			DECLARE v_dm_schema VARCHAR(20);
			DECLARE stmt VARCHAR(1000) DEFAULT ''; 
			DECLARE	v_statement_number 				INTEGER;
			DECLARE	v_module_id 							INTEGER;
			DECLARE	v_sub_module_id 					INTEGER;
			DECLARE	v_area 										VARCHAR(255);
			DECLARE	v_type 										VARCHAR(255);
			DECLARE	v_action 									VARCHAR(255);
			DECLARE	v_uow_id 									INTEGER;
			DECLARE	v_data_movement 					INTEGER;
			DECLARE	v_table_refactoring 			INTEGER;
			DECLARE	v_description 						VARCHAR(1000);
			DECLARE	v_affected_table 					VARCHAR(255);
			DECLARE v_affected_schema 				VARCHAR(255);
			DECLARE	v_object_type 						VARCHAR(255);
			DECLARE	v_object_name 						VARCHAR(255);
			DECLARE	v_pre_exists 							INTEGER;
			DECLARE	v_recoverable 						INTEGER;
			DECLARE	v_has_dependencies 				INTEGER;
			DECLARE	v_execution_status 				INTEGER;
			DECLARE	v_execute 								INTEGER;
			DECLARE	v_release 								VARCHAR(255);
			DECLARE	v_special_release 				VARCHAR(255);
			DECLARE	v_text 										VARCHAR(8192);
    	
    	SET v_action=ACTION;
    	
    	SET v_affected_schema= CASE WHEN AFFECTED_SCHEMA='RIODS' THEN ODS_SCHEMA ELSE DM_SCHEMA END;
    	SET v_affected_table=AFFECTED_TABLE;
    	SET v_object_name=OBJECT_NAME;
    	SET v_type=TYPE;    	
    	SET v_statement_number=STATEMENT_NUMBER;
    	
    	CASE  v_action
      WHEN 'ADD COLUMN' THEN
        BEGIN
       	SELECT COUNT(*) INTO v_exists FROM SYSCAT.COLUMNS WHERE TABNAME= v_affected_table AND TABSCHEMA= v_affected_schema AND COLNAME=v_object_name
       	AND TYPENAME=v_type;
       	IF (v_exists=0) THEN
       		SELECT COUNT(*) INTO v_exists FROM SYSCAT.COLUMNS WHERE TABNAME= v_affected_table AND TABSCHEMA= v_affected_schema AND COLNAME=v_object_name;
       	ELSE
       		SET v_exists=-1;	
       	END IF;
       	END;
      WHEN 'CREATE CONSTRAINT' THEN
        BEGIN
        CASE WHEN v_type='PRIMARY' THEN
      			SELECT COUNT(*) INTO v_exists FROM SYSIBM.SQLPRIMARYKEYS WHERE PK_NAME= v_object_name AND TABLE_SCHEM= v_affected_schema AND TABLE_NAME= v_affected_table;
      			ELSE
      			SELECT COUNT(*) INTO v_exists FROM SYSIBM.SQLFOREIGNKEYS WHERE FK_NAME=  v_object_name AND FKTABLE_SCHEM= v_affected_schema AND FKTABLE_NAME= v_affected_table;
      	END CASE;
      	END;
      WHEN 'CREATE INDEX' THEN
      		SELECT COUNT(*) INTO v_exists FROM SYSCAT.INDEXES WHERE INDSCHEMA=v_affected_schema and INDNAME= v_object_name ;
      WHEN 'CREATE TABLE' THEN
            SELECT COUNT(*) INTO v_exists FROM SYSCAT.TABLES WHERE TYPE='T' AND TABSCHEMA=v_affected_schema AND TABNAME = v_object_name;
      WHEN 'CREATE TRIGGER' THEN
      	  SELECT COUNT(*) INTO v_exists FROM SYSCAT.TRIGGERS WHERE TRIGSCHEMA= v_affected_schema AND TRIGNAME = v_object_name;
      WHEN 'CREATE VIEW' THEN
      	  SELECT COUNT(*) INTO v_exists FROM SYSCAT.TABLES WHERE TYPE='V' AND TABSCHEMA= v_affected_schema AND TABNAME = v_object_name;
      ELSE
        SET v_exists=0;
      END CASE;
      
      UPDATE CONFIG.STATEMENT_LIST SET PRE_EXISTS=CASE WHEN v_exists>0 THEN 1 WHEN v_exists=-1 THEN 2 ELSE 0 END WHERE STATEMENT_NUMBER=v_statement_number;
      COMMIT;
      UPDATE CONFIG.STATEMENT_LIST SET EXECUTION_STATUS=CASE WHEN PRE_EXISTS>0 THEN 2 ELSE EXECUTION_STATUS END WHERE STATEMENT_NUMBER=v_statement_number;
      COMMIT;
      RETURN;  
END@


CREATE PROCEDURE MIGRATE_MODULE(MODULE_ID INTEGER,SUB_MODULE_ID INTEGER,ODS_SCHEMA VARCHAR(20),DM_SCHEMA VARCHAR(20),TARGET_VERSION VARCHAR(10),REP_USER VARCHAR(20))
RESULT SETS 1
LANGUAGE SQL     
BEGIN

      DECLARE SQLSTATE CHAR(5) DEFAULT '00000';         
			DECLARE SQLCODE INTEGER DEFAULT 0;
			DECLARE MESSAGE_TEXT VARCHAR(100);
			DECLARE RETVAL INTEGER DEFAULT 0; 
			DECLARE v_sqlstate CHAR(5) DEFAULT '00000';
    	DECLARE v_sqlcode INTEGER DEFAULT 0;
    	DECLARE v_errmsg VARCHAR(100) DEFAULT '';
    	DECLARE err_stmt VARCHAR(1000) DEFAULT '';
    	
    	DECLARE v_target_version VARCHAR(10);
    	DECLARE v_module INTEGER;
			DECLARE v_sub_module INTEGER;
			DECLARE v_ods_schema VARCHAR(20); 
			DECLARE v_dm_schema VARCHAR(20);
			DECLARE stmt VARCHAR(1000) DEFAULT ''; 
			DECLARE	v_statement_number 				INTEGER;
			DECLARE	v_module_id 							INTEGER;
			DECLARE	v_sub_module_id 					INTEGER;
			DECLARE	v_area 										VARCHAR(255);
			DECLARE	v_type 										VARCHAR(255);
			DECLARE	v_action 									VARCHAR(255);
			DECLARE	v_uow_id 									INTEGER;
			DECLARE	v_data_movement 					INTEGER;
			DECLARE	v_table_refactoring 			INTEGER;
			DECLARE	v_description 						VARCHAR(1000);
			DECLARE	v_affected_table 					VARCHAR(255);
			DECLARE v_affected_schema 				VARCHAR(255);
			DECLARE	v_object_type 						VARCHAR(255);
			DECLARE	v_object_name 						VARCHAR(255);
			DECLARE	v_pre_exists 							INTEGER;
			DECLARE	v_recoverable 						INTEGER;
			DECLARE	v_has_dependencies 				INTEGER;
			DECLARE	v_execution_status 				INTEGER;
			DECLARE	v_execute 								INTEGER;
			DECLARE	v_release 								VARCHAR(255);
			DECLARE	v_special_release 				VARCHAR(255);
			DECLARE	v_text 										VARCHAR(8192);
			DECLARE sp_stmt                   VARCHAR(1000) DEFAULT '';
			DECLARE fetch_code CHAR(5) DEFAULT '00000';
			DECLARE v_rep_user VARCHAR(20);
			
			SET v_module=MODULE_ID;
			SET v_sub_module=SUB_MODULE_ID;
			SET v_target_version=TARGET_VERSION;
			SET v_ods_schema=ODS_SCHEMA;
			SET v_dm_schema=DM_SCHEMA;
			SET v_rep_user=REP_USER;
						
			BEGIN
			DECLARE C2 CURSOR WITH HOLD FOR 
				SELECT STATEMENT_NUMBER,MODULE_ID,SUB_MODULE_ID,AREA,TYPE,ACTION,UOW_ID,DATA_MOVEMENT,TABLE_REFACTORING,AFFECTED_TABLE,
				AFFECTED_SCHEMA,OBJECT_TYPE,OBJECT_NAME,RECOVERABLE,HAS_DEPENDENCIES,EXECUTION_STATUS,EXECUTE,RELEASE,SPECIAL_RELEASE,TEXT
				FROM CONFIG.STATEMENT_LIST 
				WHERE MODULE_ID=v_module AND SUB_MODULE_ID=v_sub_module 
				AND EXECUTE=1 AND (EXECUTION_STATUS IS NULL OR EXECUTION_STATUS<>1) AND RELEASE=v_target_version
				ORDER BY STATEMENT_NUMBER,UOW_ID;
				OPEN C2;
				FETCH C2 INTO v_statement_number,	v_module_id,v_sub_module_id,v_area,v_type,v_action,v_uow_id,v_data_movement,v_table_refactoring,v_affected_table,v_affected_schema,v_object_type,v_object_name,v_recoverable,v_has_dependencies,v_execution_status,v_execute,v_release,v_special_release,v_text;
				WHILE (SQLSTATE= '00000') DO
						SET sp_stmt= 'CALL VALIDATE_EXISTENCE(' || COALESCE(CHAR(v_statement_number),'')|| ',''' || COALESCE(v_type,'') || ''','''  || COALESCE(v_action,'')|| ''',''' || COALESCE(v_affected_table,'') || ''',''' || COALESCE(v_affected_schema,'') || ''',''' || COALESCE(v_object_type,'') || ''',''' || COALESCE(v_object_name,'') || ''',''' || COALESCE(v_ods_schema,'')|| ''',''' || COALESCE(v_dm_schema,'') || ''')';
					  PREPARE s1 FROM sp_stmt;
	        	EXECUTE s1;	
         FETCH C2 INTO v_statement_number,	v_module_id,v_sub_module_id,v_area,v_type,v_action,v_uow_id,v_data_movement,v_table_refactoring,v_affected_table,v_affected_schema,v_object_type,v_object_name,v_recoverable,v_has_dependencies,v_execution_status,v_execute,v_release,v_special_release,v_text;
				END WHILE;
				CLOSE C2;
			END;
			
			BEGIN
			DECLARE C3 CURSOR WITH HOLD FOR 
			SELECT DISTINCT MODULE_ID,SUB_MODULE_ID,UOW_ID
				FROM CONFIG.STATEMENT_LIST 
				WHERE MODULE_ID=v_module AND SUB_MODULE_ID=v_sub_module 
				AND EXECUTE=1 AND (EXECUTION_STATUS IS NULL OR EXECUTION_STATUS<>1) AND RELEASE=v_target_version AND PRE_EXISTS=0
				GROUP BY MODULE_ID,SUB_MODULE_ID,UOW_ID
				ORDER BY UOW_ID,MODULE_ID,SUB_MODULE_ID;
				OPEN C3;
				FETCH C3 INTO v_module_id,v_sub_module_id,v_uow_id;
				WHILE (SQLSTATE= '00000') DO
						SET sp_stmt='CALL PROCESS_UOW(' || COALESCE(CHAR(v_module_id),'') || ',' || COALESCE(char(v_sub_module_id),'') || ',' || COALESCE(CHAR(v_uow_id),'') || ',''' || COALESCE(v_ods_schema,'')|| ''',''' || COALESCE(v_dm_schema,'') || ''',''' || COALESCE(v_target_version,'') || ''',''' || COALESCE(v_rep_user,'') || ''')';
						PREPARE s5 FROM sp_stmt;
						EXECUTE s5;
						SET v_sqlstate=SQLSTATE;
						SET v_sqlcode=SQLCODE;
				    FETCH C3 INTO v_module_id,v_sub_module_id,v_uow_id;
    		END WHILE;
    		CLOSE C3;
    		END;
    	RETURN 0;
END@    	

CREATE PROCEDURE DW_MIGRATE(ODS_SCHEMA VARCHAR(20),DM_SCHEMA VARCHAR(20),REP_USER VARCHAR(20))
RESULT SETS 1
LANGUAGE SQL     
BEGIN

      DECLARE SQLSTATE CHAR(5) DEFAULT '00000';         
			DECLARE SQLCODE INTEGER DEFAULT 0;
			DECLARE MESSAGE_TEXT VARCHAR(100);
			DECLARE RETVAL INTEGER DEFAULT 0; 
			DECLARE v_sqlstate CHAR(5) DEFAULT '00000';
    	DECLARE v_rise INTEGER DEFAULT 0;
    	DECLARE v_errmsg VARCHAR(100) DEFAULT '';
    	DECLARE err_stmt VARCHAR(1000) DEFAULT '';
    	
    	
			DECLARE v_target_version VARCHAR(10) DEFAULT '1.0.0.2';
			DECLARE v_current_version VARCHAR(10);
			DECLARE v_migration_version VARCHAR(10);
			DECLARE v_module INTEGER;
			DECLARE v_sub_module INTEGER;
			DECLARE v_ods_schema VARCHAR(20); 
			DECLARE v_dm_schema VARCHAR(20);
			DECLARE stmt VARCHAR(1000) DEFAULT ''; 
			DECLARE sp_stmt VARCHAR(1000) DEFAULT '';
			DECLARE fetch_code CHAR(5) DEFAULT '00000';
			DECLARE  v_rep_user VARCHAR(20);

			
			DECLARE EXIT HANDLER FOR SQLSTATE '70001'                              
			BEGIN
				SET err_stmt='VALUES (' || v_dm_schema || '.SELECT_CAL_MESSAGE(''CRREW4598E''))';
				PREPARE s_err FROM err_stmt;
				BEGIN
					DECLARE CE1 CURSOR WITH HOLD FOR s_err;
					OPEN CE1;
					FETCH CE1 INTO v_errmsg;
					CLOSE CE1;
				END;                                                                                                                       
				RESIGNAL SQLSTATE '70001' SET MESSAGE_TEXT = v_errmsg;                                                                 
			END;
			
			DECLARE EXIT HANDLER FOR SQLSTATE '70002'                              
			BEGIN
				SET err_stmt='VALUES (' || v_dm_schema || '.SELECT_CAL_MESSAGE(''CRREW4599E''))';
				PREPARE s_err1 FROM err_stmt;
				BEGIN
					DECLARE CE2 CURSOR WITH HOLD FOR s_err1;
					OPEN CE2;
					FETCH CE2 INTO v_errmsg;
					CLOSE CE2;
				END;                                                                                                                       
				RESIGNAL SQLSTATE '70002' SET MESSAGE_TEXT = v_errmsg;                                                                 
			END;
			
			DECLARE EXIT HANDLER FOR SQLSTATE '70003'                              
			BEGIN
				SET err_stmt='VALUES (' || v_dm_schema || '.SELECT_CAL_MESSAGE(''CRREW4505E''))';
				PREPARE s_err2 FROM err_stmt;
				BEGIN
					DECLARE CE3 CURSOR WITH HOLD FOR s_err2;
					OPEN CE3;
					FETCH CE3 INTO v_errmsg;
					CLOSE CE3;
				END;                                                                                                                       
				RESIGNAL SQLSTATE '70003' SET MESSAGE_TEXT = v_errmsg;                                                                 
			END;
			
			DECLARE EXIT HANDLER FOR SQLSTATE '70004'                              
			BEGIN
				SET err_stmt='VALUES (' || v_dm_schema || '.SELECT_CAL_MESSAGE(''CRREW4506E''))';
				PREPARE s_err3 FROM err_stmt;
				BEGIN
					DECLARE CE4 CURSOR WITH HOLD FOR s_err3;
					OPEN CE4;
					FETCH CE4 INTO v_errmsg;
					CLOSE CE4;
				END;                                                                                                                       
				RESIGNAL SQLSTATE '70004' SET MESSAGE_TEXT = v_errmsg;                                                                 
			END;
			
			DECLARE EXIT HANDLER FOR SQLSTATE '70005'                              
			BEGIN
				SET err_stmt='VALUES (' || v_dm_schema || '.SELECT_CAL_MESSAGE(''CRREW4600E''))';
				PREPARE s_err4 FROM err_stmt;
				BEGIN
					DECLARE CE5 CURSOR WITH HOLD FOR s_err4;
					OPEN CE5;
					FETCH CE5 INTO v_errmsg;
					CLOSE CE5;
				END;                                                                                                                       
				RESIGNAL SQLSTATE '70005' SET MESSAGE_TEXT = v_errmsg;                                                                 
			END;
			
			DECLARE EXIT HANDLER FOR SQLSTATE '70006'                              
			BEGIN
				SET err_stmt='VALUES (' || v_dm_schema || '.SELECT_CAL_MESSAGE(''CRREW4543E''))';
				PREPARE s_err5 FROM err_stmt;
				BEGIN
					DECLARE CE8 CURSOR WITH HOLD FOR s_err5;
					OPEN CE8;
					FETCH CE8 INTO v_errmsg;
					CLOSE CE8;
				END;                                                                                                                       
				RESIGNAL SQLSTATE '70006' SET MESSAGE_TEXT = v_errmsg;                                                                 
			END;
			
			SET v_ods_schema=ODS_SCHEMA;
			SET v_dm_schema=DM_SCHEMA;
			SET v_rep_user=REP_USER;
			
			SELECT CHAR_VALUE INTO v_current_version FROM CONFIG.SYSCONFIG_INFO WHERE INFO_ID=6;
			
			SELECT CHAR_VALUE INTO v_target_version FROM CONFIG.SYSCONFIG_INFO WHERE INFO_ID=7;
			
			IF (v_current_version IS NULL) THEN
						SIGNAL SQLSTATE '70001';
			END IF;

			IF (v_target_version IS NULL) THEN
						SIGNAL SQLSTATE '70002';
			END IF;
			
      IF (v_current_version > v_target_version) THEN
						SIGNAL SQLSTATE '70005';
			END IF;
			
			IF (v_ods_schema IS NULL) THEN
						SIGNAL SQLSTATE '70003';
			END IF;
			
			
			IF (v_dm_schema IS NULL) THEN
						SIGNAL SQLSTATE '70004';
			END IF;
			
			IF (v_rep_user IS NULL) THEN
						SIGNAL SQLSTATE '70006';
			END IF;
			
			      
		BEGIN	
		DECLARE CVer CURSOR WITH HOLD FOR			
        SELECT DISTINCT S.RELEASE,S.MODULE_ID, S.SUB_MODULE_ID FROM CONFIG.STATEMENT_LIST S WHERE S.RELEASE >= v_current_version and S.RELEASE <= v_target_version 
				AND EXISTS(SELECT M.MODULE_ID, M.SUB_MODULE_ID FROM CONFIG.MODULES M WHERE M.MODIFY='Y' AND M.MODULE_ID=S.MODULE_ID AND M.SUB_MODULE_ID=S.SUB_MODULE_ID)
						ORDER BY S.RELEASE, S.MODULE_ID, S.SUB_MODULE_ID;						

		OPEN CVer;
		FETCH CVer INTO  v_migration_version,v_module,v_sub_module;
		WHILE (SQLSTATE= '00000') DO
				SET sp_stmt='CALL MIGRATE_MODULE('|| CHAR(v_module) || ',' || CHAR(v_sub_module) || ',''' || v_ods_schema || ''',''' || v_dm_schema || ''',''' || v_migration_version || ''',''' || v_rep_user || ''')';
				PREPARE s2 FROM sp_stmt;
				EXECUTE s2; 
				FETCH CVer INTO v_migration_version,v_module,v_sub_module;
		END WHILE; 
		CLOSE CVer;
  			
    END;			
      	UPDATE CONFIG.SYSCONFIG_INFO SET CHAR_VALUE=v_target_version WHERE INFO_ID=6;
  		-- DELETE FROM CONFIG.SYSCONFIG_INFO WHERE INFO_ID=7;

END@  

